"""
Utilities to help debugging python programs in a console easier
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from IPython.terminal import embed, ipapp


# Used by _my_embed
_embedded_shell = None


def break_here():
    import pdb
    pdb.set_trace()


def ipython_here():
    _my_embed(stack_depth=3)


def _my_embed(**kwargs):
    """
    Since we need to control sthe stack_depth, the only way is to
    copy-and-paste the implementation and change the stack_depth argument :(
    """

    config = kwargs.get('config')
    header = kwargs.pop('header', u'')
    stack_depth = kwargs.pop('stack_depth', 2)
    if config is None:
        config = ipapp.load_default_config()
        config.InteractiveShellEmbed = config.TerminalInteractiveShell
        kwargs['config'] = config

    global _embedded_shell
    if _embedded_shell is None:
        _embedded_shell = embed.InteractiveShellEmbed(**kwargs)
    _embedded_shell(header=header, stack_depth=stack_depth)



#!/usr/bin/env python
"""Prints a summary of the code file.

It makes the layout of the code to be read easily. Currently it only supports
python files.

   Copyright 2014 Herman Tai

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import re
import sys


class CodeFileSummarizer(object):
    def run(self, filepath):
        with open(filepath) as f:
            content = f.read()

        PythonFileSummarizer().summarize(content)

class PythonFileSummarizer(object):
    def summarize(self, code_in_text):
        lines = code_in_text.split('\n')
        for line in lines:
            if re.match(r"def [^ ]+\(.*\):$", line.lstrip()):
                print(line)
            elif re.match(r"def [^ ]+\($", line.lstrip()):
                # print a funciton def which spans multiple lines
                print(line + "...)")
            elif re.match(r"class [^ ]+:$", line.lstrip()):
                print(line)


def parse_args(cmd_args):
    description = """
Print a summary of the code, so the layout of the code can be read easily.
    """
    parser = argparse.ArgumentParser(
        description=description,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "file"
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    app = CodeFileSummarizer()
    app.run(args.file)


if __name__ == '__main__':
    main()



"""Utilities related to console applications


Copyright 2015 Herman Tai

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import sys

import six

from sorno import mathlib


def input(prompt, file=None):
    """
    Just like raw_input in python2 or input in python3, but can optionally
    redirect the prompt to other file objects.
    Args:
        prompt: A string for the prompt.
        file: Optional. A file object to the prompt is written to.
    """
    if file is None:
        return six.moves.input(prompt)
    else:
        old_stdout = sys.stdout
        try:
            sys.stdout = file
            return six.moves.input(prompt)
        finally:
            sys.stdout = old_stdout


class DataPrinter(object):
    PRINT_STYLE_HTML_TABLE = 'html'
    PRINT_STYLE_PLAIN = 'plain'
    PRINT_STYLE_R = 'R'
    PRINT_STYLE_TABLE = 'table'
    PRINT_STYLE_VERBOSE = 'verbose'
    NICETABLE_COL_LEN = 80
    PRINT_STYLE_R = 'R'
    PRINT_STYLE_NICETABLE = 'nicetable'

    OPTION_TITLE = 'title'
    OPTION_ADDITIONAL_TRAILING_CONTENT = 'additional-trailing-content'

    def __init__(
        self,
        data,
        headers=(),
        header_types=None,
        delimiter='\t',
        max_col_len=NICETABLE_COL_LEN,
        print_func=print,
    ):
        """Constructs a DataPrinter object

        Args:
          data: A list of dict's or a list of lists. Each dict or list
              represents a row of data. If it's a dict, the keys are the
              column names in strings and the values are the column values in
              strings.  If it's a list, then each item is the value of a
              column of the row. Each value should be a string properly
              formatted, since DataPrinter does not do any special formatting
              of each value.

          headers: Optionally supply a list of headers in strings to control
              what columns are printed out and what order are the columns
              printed.

          header_types: A list of strings representing the types of the
              headers. It's currently used for PRINT_STYLE_R form. This is not
              needed if you are not using PRINT_STYLE_R.

          delimiter: The delimiter in string for separating column values. By
              default it's a tab character.

          max_col_len: An integer to indicate the maximum column length when
              the data is printed in PRINT_STYLE_NICETABLE. By default it's
              80.

          print_func: A function that takes a string as an argument. The
              function is used for printing the data. By default it uses the
              built-in print function.
        """
        if not hasattr(data, "__getitem__"):
            data = list(data)

        if not headers and data and isinstance(data[0], dict):
            # infer headers from data
            headers = sorted(data[0].keys())

        self.headers = headers

        if data and isinstance(data[0], dict):
            self.data = self._convert_list_of_dicts(headers, data)
        else:
            self.data = data

        self.header_types = header_types
        self.delimiter = delimiter
        self.max_col_len = max_col_len
        self.print_func = print_func

    def _convert_list_of_dicts(self, headers, ls_of_dicts):
        """
        Convert list of dicts to list of lists.
        """
        data = [[d[header] for header in headers] for d in ls_of_dicts]
        return data

    def print_result(self, style=PRINT_STYLE_TABLE, options=None):
        if options is None:
            options = {}

        if style == DataPrinter.PRINT_STYLE_R:
            json_headers = []
            for header, t in zip(self.headers, self.header_types):
                h = {'name': header, 'type': t}
                json_headers.append(
                    json.dumps(h).replace('\t', '').replace('\n', '')
                    )
            self.print_func(self.delimiter.join(json_headers))
            self.print_data()
        elif style == DataPrinter.PRINT_STYLE_PLAIN:
            self.print_data()
        elif style == DataPrinter.PRINT_STYLE_VERBOSE:
            self.print_verbose()
        elif style == DataPrinter.PRINT_STYLE_HTML_TABLE:
            self.print_html_table(options)
        elif style == DataPrinter.PRINT_STYLE_NICETABLE:
            self.print_table(is_nicetable=True)
        else:
            self.print_table()

    def print_data(self):
        # Each value is a row with ^A as delimiter
        for row in self.data:
            self.print_func(self.delimiter.join(row))

    def print_table(self, is_nicetable=False):
        max_widths = [len(header) for header in self.headers]
        for row in self.data:
            for i, col in enumerate(row):
                max_widths[i] = max(max_widths[i], self.get_len(col))

        if is_nicetable:
            max_widths = [
                min(self.max_col_len, width) for width in max_widths]
            print_data_row = self.print_nice_data_row
        else:
            print_data_row = self.print_data_row

        self.print_horizontal_line(max_widths)
        # print header
        print_data_row(max_widths, self.headers)
        self.print_horizontal_line(max_widths)

        for row in self.data:
            print_data_row(max_widths, row)
        self.print_horizontal_line(max_widths)

    def print_horizontal_line(self, widths):
        segments = ['-' * (width + 2) for width in widths]
        self.print_func("+%s+" % '+'.join(segments))

    def print_data_row(self, widths, data_row):
        formatted_row = [" %-*s " % (width, col)
                            for width, col in zip(widths, data_row)]
        self.print_func("|%s|" % '|'.join(formatted_row))

    def print_verbose(self):
        if self.headers:
            max_header_len = max([len(header) for header in self.headers])
        else:
            max_header_len = 0

        for i, row in enumerate(self.data):
            self.print_func("*" * 27 + " %1d. row " % (i + 1) + "*" * 27)
            for header, col in zip(self.headers, row):
                self.print_func("%*s: %s" % (max_header_len, header, col))

    def print_nice_data_row(self, widths, data_row):
        # calculate the maximum of lines need for this row
        count_line = 1
        for width, col in zip(widths, data_row):
            col_len = self.get_len(col)
            if col_len > width:
                tmp = int(col_len / width)
                if col_len % width != 0:
                    tmp += 1
                count_line = max(count_line, tmp)

        # print them out
        for i in range(count_line):
            cells = []
            for width, col in zip(widths, data_row):
                col_len = self.get_len(col)
                spaces_used = i * width
                if col_len > spaces_used:
                    # We display the maximum number of characters for this
                    # column, or the number of characters still left to
                    # display if it's smaller
                    len_to_display = min(col_len - width * i, width)
                    text = col[spaces_used:spaces_used + len_to_display]
                    cells.append(text.replace('\t', ' ').replace('\n', ' '))
                else:
                    cells.append("")
            formatted = [" %-*s " % (width, col)
                            for width, col in zip(widths, cells)]
            self.print_func("|%s|" % '|'.join(formatted))

    def print_html_table(self, options):
        # print everything before the table
        html = """
<html>
<head>
    <title>{title}</title>
    <style>
    table {{
        border-collapse:collapse;
    }}
    table, tr, td, th {{
        border: 1px solid black;
        padding: 3px;
    }}
    th {{
        background: #333;
        color: white;
        font-size: 80%;
    }}
    tr:nth-child(even) {{
        background: #ccc;
    }}
    tr:nth-child(odd) {{
        background: #fff;
    }}

    </style>
</head>
<body>
<table style="border: 1px solid black">
{headers}
{rows}
</table>
{trailing_content}
<p>Created at {timestamp}</p>
</body>
</html>
        """

        title = options.get(self.OPTION_TITLE, "")
        trailing_content = options.get(
            self.OPTION_ADDITIONAL_TRAILING_CONTENT,
            "",
        )
        headers = "<tr><th>" + "</th><th>".join(self.headers) + "</th></tr>\n"

        html_rows = []
        for row in self.data:
            partial_html_row = "</td><td>".join(row)
            html_row = "<tr><td>" + partial_html_row + "</td></tr>"
            html_rows.append(html_row)

        rows = "\n".join(html_rows)

        timestamp = time.ctime()

        self.print_func(html.format(**locals()))

    def get_len(self, value):
        if not value:
            return 0
        else:
            return len(value)


def parse_intervals(s):
    """
    Parse a string to return the intervals the string represents.

    Example:
        >>> parse_intervals("1,2")
        [Interval(start=1,end=1,is_start_opened=False,is_end_opened=False), Interval(start=2,end=2,is_start_opened=False,is_end_opened=False)]
    """
    s = s.strip()
    if not s:
        return []

    interval_strs = [segment.strip() for segment in s.split(',')]
    intervals = []
    for interval_str in interval_strs:
        num_or_interval = interval_str.split('-', 1)
        if len(num_or_interval) == 1:
            s = e = int(num_or_interval[0])
        else:
            s = int(num_or_interval[0])
            e = int(num_or_interval[1])

        interval = mathlib.Interval(
            s,
            e,
            is_start_opened=False,
            is_end_opened=False,
        )
        intervals.append(interval)

    return intervals


def confirm(prompt):
    """Ask the console application user a yes/no question.

    The user is expected to enter "y" or "t" for Yes (True), and "n" or "f"
    for No (False). The response is case-insensitive.

    "yes", "no", "true" and "false" works as expected.

    If the user enters other values, the question is repeated.

    Args:
        prompt (str): The question to ask the user.
    Returns:
        bool: True if the user's response means Yes, False if the user's
            response means No.
    """
    while True:
        ans = six.moves.input(prompt + " [y/N/t/F] ").lower()

        if ans in ("y", "yes", "t", "true"):
            return True
        if ans in ("n", "no", "f", "false"):
            return False


if __name__ == "__main__":
    rows = [
        {'a': "apple", 'b': "boy"},
        {'a': "one", 'b': "two"},
    ]
    DataPrinter(rows).print_result()



"""Utilities to work with the standard logging library.

Most of the time, you simply need to do the following in the file containing
your main function (the entry point of your program):

    import logging
    from sorno import loggingutil

    _log = logging.getLogger()  # to get the root logger
    loggingutil.setup_logger(_log)

This sets a reasonable useful format for logging messages. See the doc of the
setup_logger function for more options.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import logging
from logging.handlers import TimedRotatingFileHandler
import os
import subprocess
import sys


def setup_logger(
    logger,
    debug=False,
    stdout=False,
    log_to_file=None,
    add_thread_id=False,
    logging_level=None,
    use_path=False,
    suppress_stream_handler_broken_pipe_error=True,
):
    """Setup a given logger with the parameters given.

    The logger being edited always outputs to stdout or stderr with
    logging.StreamHandler in the standard logging module in additional to
    other handlers.

    Args:
        logger: The standard module logging.Logger instance to be edited
        debug: A boolean. True if the logger should be set to debug level.
            This argument is overridden if a non-None argument is passed to
            the logging_level parameter.
        stdout: A boolean to indicate the logger should use stdout or stderr
            for the logging.StreamHandler
        log_to_file: A filepath for a file being logged to.
            logging.handlers.TimedRotatingFileHandler and a new file is used
            daily with the old file renamed with the date.
        add_thread_id: A boolean. True if the logging format string should
            include the thread id.
        use_path: A boolean. True if the logging format string should use the
            file path name instead of the module name.
        suppress_stream_handler_broken_pipe_error: A boolean. True if
            suppressing the broken pipe error caused by flusing the stream in
            logging.StreamHandler. It's usually because of the output is
            getting piped to the unix "head" command on other commands that
            closes the standard input pipe prematurely.

    Returns:
        Nothing, since the logger is edited in-place.
    """
    if logging_level is None:
        if debug:
            logging_level = logging.DEBUG
        else:
            logging_level = logging.INFO

    formatter = create_logging_formatter(
        add_thread_id=add_thread_id,
        use_path=use_path
    )
    hdlr = create_stream_handler(
        formatter=formatter,
        stdout=stdout,
        suppress_stream_handler_broken_pipe_error=(
            suppress_stream_handler_broken_pipe_error
        ),
    )

    logger.handlers = []  # clear the existing handlers
    logger.addHandler(hdlr)
    logger.setLevel(logging_level)

    if log_to_file is not None:
        init_command = 'mkdir -p %s' % os.path.dirname(log_to_file)

        subprocess.check_call(init_command, shell=True)

        hdlr = TimedRotatingFileHandler(
            log_to_file,
            when="midnight",
            interval=1,
        )
        hdlr.setFormatter(formatter)
        logger.addHandler(hdlr)


def suppress_stream_handler_broken_pipe_error_in_flush(stream_handler):
    old_flush = stream_handler.flush
    def wrapper(*args, **kwargs):
        try:
            old_flush(*args, **kwargs)
        except IOError as e:
            if e.errno == 32:
                return
            else:
                raise
    stream_handler.flush = wrapper


def create_plain_logger(
    logger_name,
    debug=False,
    stdout=True,
    logging_level=None,
    suppress_stream_handler_broken_pipe_error=True,
):
    plain_logger = logging.getLogger(logger_name)
    plain_logger.propagate = False

    if logging_level is None:
        if debug:
            logging_level = logging.DEBUG
        else:
            logging_level = logging.INFO

    plain_logger.setLevel(logging_level)

    if stdout:
        out = sys.stdout
    else:
        out = sys.stderr

    formatter = logging.Formatter(
        fmt="%(message)s",
        datefmt="%Y",  # does not matter
    )
    handler = create_stream_handler(
        formatter=formatter,
        stdout=stdout,
        suppress_stream_handler_broken_pipe_error=(
            suppress_stream_handler_broken_pipe_error
        ),
    )

    plain_logger.addHandler(handler)
    return plain_logger


def create_logging_formatter(add_thread_id=False, use_path=False):
    format_str = "%(asctime)s"

    if add_thread_id:
        format_str += " thread:%(thread)s"

    format_str += " %(levelname)s "

    if use_path:
        format_str += "%(pathname)s"
    else:
        format_str += "%(name)s"

    format_str += ":%(lineno)s: %(message)s"

    detail_formatter = logging.Formatter(
        fmt=format_str,
        datefmt="%Y-%m-%d %H:%M:%S"
    )
    return detail_formatter


def create_stream_handler(
    formatter=None,
    stdout=False,
    suppress_stream_handler_broken_pipe_error=True,
):
    if formatter is None:
        formatter = create_logging_formatter()

    if stdout:
        stream = sys.stdout
    else:
        stream = sys.stderr

    hdlr = logging.StreamHandler(stream=stream)
    hdlr.setFormatter(formatter)

    if suppress_stream_handler_broken_pipe_error:
        suppress_stream_handler_broken_pipe_error_in_flush(hdlr)
    return hdlr


if __name__ == '__main__':
    plain_stdout_logger = create_plain_logger(
        "plain_logger",
        stdout=True,
    )
    plain_stdout_logger.info("plain stdout logger at info level")
    plain_stdout_logger.debug("should not see this one")

    plain_stdout_debug_logger = create_plain_logger(
        "plain_debug_logger",
        stdout=True,
        debug=True,
    )
    plain_stdout_debug_logger.info("plain stdout debug logger at info level")
    plain_stdout_debug_logger.debug("plain stdout debug logger at debug level")

    plain_stderr_logger = create_plain_logger(
        "plain_stderr_logger",
        stdout=False,
    )
    plain_stderr_logger.info("plain stderr logger at info level")
    plain_stderr_logger.debug("should not see this one")



#!/usr/bin/env python
"""Saves images with reduced sizes.

Reduces the sizes of all images in a directory and its subdirectories by
saving them with lower quality jpg format. The directory structure is
preserved but the new directory is created with a timestamp suffix.


    Copyright 2014 Herman Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import os
import sys
import time

from PIL import Image


_LOG = logging.getLogger(__name__)


class App(object):
    def __init__(
        self,
        src_dir,
        dest_dir,
        dry_run=False,
    ):
        self.src_dir = src_dir
        self.dest_dir = dest_dir
        self.real_src_dirpath = os.path.realpath(src_dir)
        self.dry_run = dry_run

    def run(self):
        _LOG.info("Create directory %s", self.dest_dir)
        if not self.dry_run:
            os.mkdir(self.dest_dir)

        for root, dirs, files in os.walk(self.src_dir):
            for d in dirs:
                dirpath = os.path.realpath(os.path.join(root, d))
                dest_path = self.make_dest_path(dirpath)
                _LOG.info(
                    "Create directory %s",
                    dest_path,
                )

                if not self.dry_run:
                    os.mkdir(dest_path)

            for f in files:
                filepath = os.path.realpath(os.path.join(root, f))
                ext = os.path.splitext(filepath)[1]
                if ext in (".jpg", ".png"):
                    dest_path = self.make_dest_path(filepath)
                    if not dest_path.endswith(".jpg"):
                        dest_path = os.path.splitext(dest_path)[0] + ".jpg"

                    _LOG.info(
                        "Shrink the size of %s and save to %s",
                        filepath,
                        dest_path,
                    )
                    src_image = Image.open(filepath)
                    if not self.dry_run:
                        src_image.save(
                            dest_path,
                            optimize=True,
                            quality=85,
                        )

                        src_size, src_atime, src_mtime = os.stat(filepath)[
                            6:9
                        ]
                        dest_size = os.path.getsize(dest_path)
                        _LOG.info(
                            "Saved. Size is reduced from %s bytes to %s bytes",
                            src_size,
                            dest_size,
                        )
                        _LOG.info(
                            "Change the atime and mtime for %s to %s and %s",
                            dest_path,
                            src_atime,
                            src_mtime,
                        )
                        os.utime(dest_path, (src_atime, src_mtime))


    def make_dest_path(self, src_path):
        if not src_path.startswith(self.real_src_dirpath):
            raise Exception(
                "{0} does not start with {1}".format(
                    src_path,
                    self.real_src_dirpath,
                )
            )

        return os.path.join(
            self.dest_dir,
            src_path[len(self.real_src_dirpath):].lstrip('/'),
        )

#
# logger-related functions
#

def setup_logger(
    logger,
    debug=False,
    stdout=False,
    add_thread_id=False,
    logging_level=None,
    use_path=False,
):
    if logging_level is None:
        if debug:
            logging_level = logging.DEBUG
        else:
            logging_level = logging.INFO

    formatter = create_logging_formatter(
        add_thread_id=add_thread_id,
        use_path=use_path
    )
    hdlr = create_stream_handler(
        formatter=formatter,
        stdout=stdout,
    )
    logger.handlers = []  # clear the existing handlers
    logger.addHandler(hdlr)
    logger.setLevel(logging_level)


def create_logging_formatter(add_thread_id=False, use_path=False):
    format_str = "%(asctime)s"

    if add_thread_id:
        format_str += " thread:%(thread)s"

    format_str += " %(levelname)s "

    if use_path:
        format_str += "%(pathname)s"
    else:
        format_str += "%(name)s"

    format_str += ":%(lineno)s: %(message)s"

    detail_formatter = logging.Formatter(
        fmt=format_str,
        datefmt="%Y-%m-%d %H:%M:%S"
    )
    return detail_formatter


def create_stream_handler(formatter=None, stdout=False):
    if formatter is None:
        formatter = create_logging_formatter()

    if stdout:
        stream = sys.stdout
    else:
        stream = sys.stderr

    hdlr = logging.StreamHandler(stream=stream)
    hdlr.setFormatter(formatter)
    return hdlr

#
# end of logger-related functions
#

def parse_args(cmd_args):
    description = """
Reduce the sizes of all images in a directory and its subdirectories by saving them with lower
quality jpg format. The directory structure is preserved but the new directory
is created with a timestamp suffix.
    """
    parser = argparse.ArgumentParser(
        description=description,
    )
    parser.add_argument(
        "--dry-run",
        action="store_true",
    )
    parser.add_argument(
        "src_dir",
    )
    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    setup_logger(_LOG)

    dest_dir = args.src_dir.rstrip('/') + time.strftime("_%Y_%m_%d_%H_%M_%S")

    app = App(
        args.src_dir,
        dest_dir,
        dry_run=args.dry_run,
    )
    app.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""A script to prompt for choosing items generated from different sources, then
print those items out. You can then use those items for other shell commands.

Examples:

    If you have a script to generate common directories that you use, e.g.
    gen-fav-dir.sh, you can put the following in your .bashrc, assuming
    sorno_pick.py and gen-fav-dir.sh are in your PATH:

        $ alias cdf='cd $(sorno_pick.py -c gen-fav-dir.sh)'

    Then you can just type:

        $ cdf

    And you will be given a list of directories to "cd" to.

    P.S. You probably want to set the alias to the following:

        $ alias cdf='tmp="cd $(sorno_pick.py -c gen-fav-dir.sh)";history -s \\
            "$tmp";$tmp'

    This ensures the history is inserted in a useful way, e.g. when you run
    "history" or press the "up" key, you see the actual command "cd xxx" instead
    of just "cdf".

    I personally have a few template files, so I have a file to store the
    directories containing my template files, then I put the following
    function in my bashrc file:

        function copy-from-paths() {
          local dest_file=$1
          shift
          cp $(sorno_pick.py --filepaths-from-file \\
              ~/path/to/copy-from-paths.txt "$@") "$dest_file"
        }

    Notice the use of "$@", that allows me to add options to sorno_pick.py,
    e.g. -r '\\.py$' to only get template files for python. The whole coomand
    is:

        copy-from-paths /tmp/tmpfile -r '\.py$'

    Similarily, I have the following function for vim:

        function vic {
          local tmp="vim -O $(sorno_pick.py -c generate_files.py \\
            --output-delimiter ' ' $@)"
          history -s "$tmp"
          $tmp
        }


    Copyright 2014 Herman Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import os
import re
import sys
import subprocess


from sorno import loggingutil
from sorno import consoleutil


_LOG = logging.getLogger(__name__)


class PickerApp(object):
    def __init__(
        self,
        commands=None,
        filepaths=None,
        filepaths_from_filepaths=None,
        regexes=None,
        exclude_regexes=None,
        output_delimiter=os.linesep,
        output_items_with_path_expansions=False,
        enumerate_items=True,
        list_only=False,
        is_lucky=False
    ):
        self.commands = commands
        self.filepaths = filepaths
        self.filepaths_from_filepaths = filepaths_from_filepaths
        self.regexes = regexes
        self.exclude_regexes = exclude_regexes
        self.output_delimiter = output_delimiter
        self.output_items_with_path_expansions = (
            output_items_with_path_expansions
        )
        self.enumerate_items = enumerate_items
        self.list_only = list_only
        self.is_lucky = is_lucky

    def run(self):
        items = []
        items.extend(self.get_items_from_commands())
        items.extend(self.get_items_from_filepaths())
        items.extend(self.get_items_from_filepaths_from_filepaths())

        items = self.filter_out_items(items)

        chosen_items = self.pick_items(items)
        self.print_chosen_items(chosen_items)

    def pick_items(self, items):
        items = list(set(items))
        items.sort()

        if self.enumerate_items:
            for i, item in enumerate(items, 1):
                print("%d)" % i, item, file=sys.stderr)
        else:
            for item in items:
                print(item, file=sys.stderr)

        if self.list_only:
            return []

        if self.is_lucky:
            if items:
                return [items[0]]
            else:
                return []

        reply = consoleutil.input("Please choose:", file=sys.stderr)

        num_strs = [s.strip() for s in reply.split(',')]
        nums = []
        for s in num_strs:
            nums.extend(self.num_str_to_nums(s))

        chosens = [items[num - 1] for num in nums]
        return chosens

    def num_str_to_nums(self, num_str):
        if num_str:
            return [int(num_str)]
        else:
            return []

    def print_chosen_items(self, chosen_items):
        if self.output_items_with_path_expansions:
            _LOG.debug("Items are expanded with path expansions")
            chosen_items = [
                self.expand_path(item)
                for item in chosen_items
            ]

        if chosen_items:
            print(self.output_delimiter.join(chosen_items))

    def filter_out_items(self, items):
        if not self.regexes and not self.exclude_regexes:
            return items


        if self.regexes:
            remaining_items = []
            for item in items:
                matched = True
                for regex in self.regexes:
                    if not re.search(regex, item):
                        _LOG.debug(
                            "Item [%s] does not match the regex [%s]",
                            item,
                            regex,
                        )
                        matched = False
                        break

                if matched:
                    remaining_items.append(item)

            items = remaining_items

        if self.exclude_regexes:
            remaining_items = []
            regex = "|".join(["(" + r + ")" for r in self.exclude_regexes])
            _LOG.debug("Regex to be used for exclusion: %s", regex)

            for item in items:
                if re.search(regex, item):
                    _LOG.debug("Item [%s] is excluded", item)
                else:
                    remaining_items.append(item)

            items = remaining_items

        return items

    def get_items_from_commands(self):
        items = []
        if not self.commands:
            return items

        for command in self.commands:
            _LOG.debug("Execute command [%s] to get items", command)
            output = subprocess.check_output(command, shell=True)
            for line in output.split(os.linesep):
                line = line.rstrip(os.linesep)
                if line:
                    items.append(line)

        return items

    def get_items_from_filepaths(self):
        items = []
        if not self.filepaths:
            return items

        for filepath in self.filepaths:
            _LOG.debug("Read items from file [%s]", filepath)
            with open(filepath) as f:
                for line in f:
                    items.append(line.strip())

        return items

    def get_items_from_filepaths_from_filepaths(self):
        items = []
        if not self.filepaths_from_filepaths:
            return items

        for filepath in self.filepaths_from_filepaths:
            _LOG.debug("Read filepaths from file [%s]", filepath)
            with open(filepath) as f:
                for line in f:
                    items.extend(
                        self.retrieve_filepaths(self.expand_path(line.strip()))
                    )

        return items

    def retrieve_filepaths(self, filepath):
        filepaths = []
        if os.path.exists(filepath):
            if os.path.isdir(filepath):
                for root, dirs, files in os.walk(filepath):
                    for filename in files:
                        filepaths.append(os.path.join(root, filename))
            else:
                filepaths.append(filepath)

        return filepaths

    def expand_path(self, path):
        return os.path.expandvars(os.path.expanduser(path))


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2014")[0].rstrip()
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument(
        "--debug",
        action="store_true",
    )

    item_sources_options = parser.add_argument_group('Item sources')
    item_sources_options.add_argument(
        "-c", "--command",
        help="Shell commands to generate items to be picked from."
            + " The commands are invoked in a subshell and should print out"
            + " one item per line. Empty lines are filtered out",
        action="append",
    )
    item_sources_options.add_argument(
        "-f", "--file",
        help="Files that contain items to be picked from. Each line of the"
            + " file contains one item. Leading and trailing spaces are"
            + " removed and empty lines are filtered out.",
        action="append",
    )
    item_sources_options.add_argument(
        "--filepaths-from-file",
        help="Treat each item of a file as a filepath or a directory path."
            + " If it's a filepath, it's listed to be picked as is if the"
            + " file exists. If it's"
            + " a directory path, the files inside the directory are used"
            + " instead of the directory path, recursively.",
        action="append",
    )
    item_sources_options.add_argument(
        "-e", "--exclude-regex",
        help=re.sub(r"\s+", " ", """
            Items matching the regex are not shown in the list to be picked.
        """),
        action="append",
    )
    item_sources_options.add_argument(
        "-r", "--regex",
        help=re.sub(r"\s+", " ", """
            Only items matching the regex are shown in the list to be picked.
        """),
        action="append",
    )

    output_options = parser.add_argument_group("Output options")

    output_options.add_argument(
        "--output-delimiter",
        default=os.linesep,
        help="The delimiter for the choosen items printed out. By default,"
            + "the OS specific line separating character(s) is used.",
    )
    output_options.add_argument(
        "--output-items-with-path-expansions",
        help="When printing out the items, treats them as paths with"
        + " shell-like path expansion. This includes: \"~\", \"~user\", and "
        + " environment variables denoted by \"$var\" or \"${var}\"",
        action="store_true",
    )
    output_options.add_argument(
        "--no-enumerate",
        help="Not enumerating the items.",
        dest="enumerate_items",
        action="store_false",
        default=True,
    )

    parser.add_argument(
        "-l", "--list-only",
        help="Not prompting for choosing items. Only listing them.",
        action="store_true",
    )

    parser.add_argument(
        "--lucky",
        action="store_true",
        help="Choose the first choice without prompting",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_LOG, debug=args.debug)

    app = PickerApp(
        commands=args.command,
        filepaths=args.file,
        filepaths_from_filepaths=args.filepaths_from_file,
        regexes=args.regex,
        exclude_regexes=args.exclude_regex,
        output_delimiter=args.output_delimiter,
        output_items_with_path_expansions=(
            args.output_items_with_path_expansions
        ),
        enumerate_items=args.enumerate_items,
        list_only=args.list_only,
        is_lucky=args.lucky,
    )
    app.run()


if __name__ == '__main__':
    main()



#!/usr/bin/python
"""A command line client for Google Docs.

The API doc used to implement this is in
https://developers.google.com/drive/web/quickstart/quickstart-python

You can search for a file and download its content (only if it's a doc).

In order to use this script, please look at the "Using scripts involve Google
App API" section of the sorno-py-scripts README (can be found in
https://github.com/hermantai/sorno-scripts/tree/master/sorno-py-scripts). The
API needed for this script is "Drive API" with the scope
'https://www.googleapis.com/auth/drive.readonly'


    Copyright 2014 Herman Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import argparse
import os
import pprint
import subprocess
import sys

import apiclient
from apiclient.discovery import build
import httplib2
from oauth2client.client import OAuth2WebServerFlow
from oauth2client.file import Storage


# Check https://developers.google.com/drive/scopes for all available scopes
OAUTH_SCOPE = 'https://www.googleapis.com/auth/drive.readonly'

# Redirect URI for installed apps
REDIRECT_URI = 'urn:ietf:wg:oauth:2.0:oob'

CREDENTIALS_FILE = os.path.expanduser("~/.sorno_gdoc-google-drive-api.cred")


def _main():
    args = _parse_args(sys.argv[1:])

    query = args.query
    output_file = args.output

    # Copy your credentials from the console
    client_id = os.getenv('GOOGLE_APP_PROJECT_CLIENT_ID')
    client_secret = os.getenv('GOOGLE_APP_PROJECT_CLIENT_SECRET')

    if not client_id:
        print(
            "Please set the environment variable"
            " GOOGLE_APP_PROJECT_CLIENT_ID")
        sys.exit(1)

    if not client_secret:
        print(
            "Please set the environment variable"
            " GOOGLE_APP_PROJECT_CLIENT_SECRET"
        )
        sys.exit(1)

    # Run through the OAuth flow and retrieve credentials
    flow = OAuth2WebServerFlow(client_id, client_secret, OAUTH_SCOPE,
                               redirect_uri=REDIRECT_URI)

    # Indicate we need the user to grant us permissions and give the auth code or
    # not
    need_get_code = True

    if os.path.exists(CREDENTIALS_FILE) and args.use_credentials_cache:
        storage = Storage(CREDENTIALS_FILE)
        credentials = storage.get()
        print("Use old credentials")
        need_get_code = False

    if need_get_code:
        authorize_url = flow.step1_get_authorize_url()
        print 'Go to the following link in your browser: ' + authorize_url
        code = raw_input('Enter verification code: ').strip()

        credentials = flow.step2_exchange(code)

        storage = Storage(CREDENTIALS_FILE)
        storage.put(credentials)

    # Create an httplib2.Http object and authorize it with our credentials
    http = httplib2.Http()
    http = credentials.authorize(http)

    drive_service = build('drive', 'v2', http=http)

    results = retrieve_all_files(drive_service, query)

    if not results:
        print("No results")
        sys.exit(1)
    elif len(results) != 1:
        for i, res in enumerate(results):
            print("%d: %s" % (i, res['title']))

        if args.lucky:
            chosen = 0
        elif args.lucky_number:
            chosen = int(args.lucky_number) - 1  # from 1-based to 0-based
        else:
            chosen = int(raw_input("Choose file: "))

        chosen_file = results[chosen]
        print("Chosen: %s" % chosen_file['title'])
    else:
        chosen_file = results[0]

    pprint.pprint(chosen_file)

    print("File path: ")
    path = []
    parents = chosen_file['parents']
    while parents:
        parent = parents[0]
        parent_file = drive_service.files().get(fileId=parent['id']).execute()
        path.append(parent_file['title'])
        parents = parent_file['parents']

    path.reverse()
    path.append(chosen_file['title'])

    print(" > ".join(path))

    if args.metadata_only:
        return

    links = chosen_file.get('exportLinks')

    if not links or "text/plain" not in links:
        print("No text/plain links available")
        sys.exit(1)

    content = download_file(drive_service, links['text/plain'])

    if output_file:
        with open(output_file, "w") as f:
            f.write(content)
        print("The file content is written in [%s]" % output_file)

        if args.open_with:
            subprocess.check_call(
                args.open_with + " " + output_file,
                shell=True,
            )
    else:
        if args.open_with:
            p = subprocess.Popen(args.open_with, stdin=subprocess.PIPE, shell=True)
            p.communicate(content)
            p.wait()
        else:
            print(content)


def _parse_args(cmd_args):
    description = __doc__.split("Copyright 2014")[0].strip()
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument(
        "query",
        help="The search query, which only applies on the titles of the files",
    )

    parser.add_argument(
        "--lucky",
        action="store_true",
        help="Choose the first result from the query without prompting",
    )

    parser.add_argument(
        "--lucky-number",
        help="Choose the lucky-number'th result instead of the first without"
            " prompting",
    )

    parser.add_argument(
        "--metadata-only",
        action="store_true",
        help="Only show the metadata of the file",
    )

    parser.add_argument(
        "--no-credentials-cache",
        dest="use_credentials_cache",
        action="store_false",
        default=True,
        help="If specified, old credentials are not reused and you have to"
            " follow the instruction from this script to get the code every"
            " time you use this script.",
    )

    parser.add_argument(
        "--open-with",
        help="This option can be used to run the command"
        " specified with the filepath being the first command line argument"
        " to invoke the command after the file is written if --output is"
        " specified. If --output is not specified, the standard input of"
        " the process created by the command is filled with the content of"
        " the file instead."
        " Some useful commands for this option are less, sed, cat, etc."
    )

    parser.add_argument(
        "--output",
        help="Output to a file instead of printing the content of the file"
            "out",
    )

    args = parser.parse_args(cmd_args)
    return args


def download_file(service, download_url):
  """Download a file's content.

  Args:
    service: Drive API service instance.
    drive_file: Drive File instance.

  Returns:
    File's content if successful, None otherwise.
  """
  resp, content = service._http.request(download_url)
  if resp.status == 200:
      print 'Status: %s' % resp
      return content
  else:
      print 'An error occurred: %s' % resp
      return None


def retrieve_all_files(service, query):
  """Retrieve a list of File resources.

  Args:
    service: Drive API service instance.
  Returns:
    List of File resources.
  """
  result = []
  page_token = None
  while True:
      try:
          param = {'q': "title contains '%s'" % query}
          if page_token:
              param['pageToken'] = page_token
          files = service.files().list(**param).execute()

          result.extend(files['items'])
          page_token = files.get('nextPageToken')
          if not page_token:
              break
      except apiclient.errors.HttpError, error:
          print 'An error occurred: %s' % error
          break
  return result


if __name__ == '__main__':
    _main()




from __future__ import print_function

from xml.dom import minidom

def prettify(xml_str, indent=2):
    parsed = minidom.parseString(xml_str)
    return parsed.toprettyxml(indent=" " * indent)

if __name__ == '__main__':
    print(
        prettify("<fruits><fruit>apple</fruit><fruit>orange</fruit></fruits>")
    )



#!/usr/bin/env python
"""A command line client for Google Tasks

It's done by following the tutorials in Google Developers:
https://developers.google.com/google-apps/tasks/quickstart/python.

In order to use this script, please look at the "Using scripts involve Google
App API" section of the sorno-py-scripts README (can be found in
https://github.com/hermantai/sorno-scripts/tree/master/sorno-py-scripts). The
API needed for this script is "Tasks API" with the scope
'https://www.googleapis.com/auth/tasks'.

Examples:

    To print tasks for all of your task lists:

        $ sorno_gtasks.py get_tasks

    To print task only for your task list "list1" and "list2":

        $ sorno_gtasks.py get_tasks list1 list2


    Copyright 2014 Herman Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import httplib2
import logging
import os
import pprint
import re
import subprocess
import sys

from apiclient.discovery import build
from oauth2client.client import OAuth2WebServerFlow
from oauth2client.file import Storage
from oauth2client import tools

from sorno import consoleutil
from sorno import loggingutil


# The oauth scope needed for Google Tasks API
OAUTH_SCOPE = 'https://www.googleapis.com/auth/tasks'

# The file path that stores the access token returned by Google from oauth
# authentication
CREDENTIALS_FILE = os.path.expanduser("~/.sorno_gtasks-google-drive-api.cred")

_log = logging.getLogger()
_plain_logger = None  # will be created in main()
_plain_error_logger = None  # will be created in main()


class GoogleTasksConsoleApp(object):
    """The controller of the sorno_gtasks script"""

    # error codes
    EXIT_CODE_USER_ABORT = 2
    EXIT_CODE_USER_INPUT_ERROR = 3

    def __init__(
        self,
    ):
        self.tasks_service = None

    def auth(self, flags, use_credentials_cache=True):
        """
        Authenticates either by an existing credentials or by prompting the
        user to grant permissions. If succeeds, set self.tasks_service to the
        service client that can call tasks api. Otherwise, it aborts the
        script.

        Args:
            flags (argparse.Namespace): The flags for this script
            use_credentials_cache (Optional[bool]): If true, uses the
                credentials stored in ``CREDENTIALS_FILE``.
        """
        # Copy your credentials from the console
        client_id = os.getenv('GOOGLE_APP_PROJECT_CLIENT_ID')
        client_secret = os.getenv('GOOGLE_APP_PROJECT_CLIENT_SECRET')

        if not client_id:
            _log.info(
                "Please set the environment variable"
                " GOOGLE_APP_PROJECT_CLIENT_ID"
            )
            sys.exit(1)

        if not client_secret:
            _log.info(
                "Please set the environment variable"
                " GOOGLE_APP_PROJECT_CLIENT_SECRET"
            )
            sys.exit(1)

        # Run through the OAuth flow and retrieve credentials
        flow = OAuth2WebServerFlow(
            client_id,
            client_secret,
            OAUTH_SCOPE,
        )

        # Indicate we need the user to grant us permissions and give the auth
        # code or not
        need_get_code = True

        storage = Storage(CREDENTIALS_FILE)
        if os.path.exists(CREDENTIALS_FILE) and use_credentials_cache:
            credentials = storage.get()
            _log.debug("Use old credentials")
            need_get_code = False

        if need_get_code:
            credentials = tools.run_flow(flow, storage, flags)

        # Create an httplib2.Http object and authorize it with our credentials
        http = httplib2.Http()
        http = credentials.authorize(http)

        self.tasks_service = build('tasks', 'v1', http=http)

    def copy_tasks_action(self, args):
        self.auth(args, use_credentials_cache=args.use_credentials_cache)

        # ask the user to choose a task list that contains the tasks

        tasklists = self.get_tasklists()
        for index, tasklist in enumerate(tasklists, 1):
            print(
                "{0}) {1} (id: {2})".format(
                    index,
                    tasklist['title'],
                    tasklist['id'],
                )
            )

        ans = consoleutil.input(
            "Please choose the list that contains the tasks: "
        )

        intvs = consoleutil.parse_intervals(ans)
        list_number = intvs[0].start
        chosen_tasklist = tasklists[list_number - 1]

        _plain_logger.info("")
        # ask the user to choose the tasks from the chosen task list

        tasks = self.get_tasks_from_tasklist(chosen_tasklist['id'])

        _plain_logger.info(
            "Tasklist [%s] has the following tasks:",
            chosen_tasklist['title'],
        )
        self._print_tasks_with_ids(tasks)

        ans = consoleutil.input(
            "Please choose the tasks that you want to copy over: "
        )

        intvs = consoleutil.parse_intervals(ans)

        chosen_tasks = []
        for intv in intvs:
            chosen_tasks.extend(
                # the start and end in interals are one-based, so need to
                # reduce them each by 1
                tasks[intv.start - 1:intv.end]
            )

        dups = self._get_duplicated_items(chosen_tasks)
        if dups:
            _plain_logger.error(
                "The following tasks are chosen more than once:"
            )
            self._print_tasks_with_ids(dups, ref=tasks)
            return GoogleTasksConsoleApp.EXIT_CODE_USER_INPUT_ERROR

        _plain_logger.info("Chosen tasks:")
        self._print_tasks_with_ids(chosen_tasks, ref=tasks)

        _plain_logger.info("")
        # ask the user to choose the destination task list

        ans = consoleutil.input(
            "Please choose the destination task list:"
        )

        intvs = consoleutil.parse_intervals(ans)
        dest_list_number = intvs[0].start
        chosen_dest_tasklist = tasklists[dest_list_number - 1]

        if chosen_tasklist == chosen_dest_tasklist:
            _plain_error_logger.error(
                "Source and destination lists cannot be the same."
            )
            return GoogleTasksConsoleApp.EXIT_CODE_USER_INPUT_ERROR

        _plain_logger.info("")
        # ask the user to confirm
        confirm_msg = re.sub(
            r"\s+",
            " ",
            """
            Are you sure you want to copy the chosen tasks from task list
            [{0}] to task list [{1}]
            """.strip().format(
                chosen_tasklist['title'],
                chosen_dest_tasklist['title'],
            ),
        )

        if not consoleutil.confirm(confirm_msg):
            _plain_error_logger.error("Aborted")
            return GoogleTasksConsoleApp.EXIT_CODE_USER_ABORT

        # copy the tasks

        for task in chosen_tasks:
            self.insert_task(
                chosen_dest_tasklist['id'],
                task,
            )

        return 0

    def insert_task(self, tasklist_id, task):
        """Insert a task

        Args:
            tasklist_id (str): The tasklist that contains the task
            task (dict): The representation of a task. See
                https://developers.google.com/google-apps/tasks/v1/reference/tasks#resource-representations
        """
        t = task.copy()
        t.pop('id', ' ')
        self.tasks_service.tasks().insert(
            tasklist=tasklist_id,
            body=t,
        ).execute()

    def _print_tasks_with_ids(self, tasks, ref=None):
        """Print tasks along with their id's.

        Args:
            tasks (sequence[task]): Tasks to be printed out
            ref (Optional[sequence[task]]): By default, this method simply
                prints the sequence numbers of the tasks according to the
                position of the task in the given tasks (one-based). If ref is
                provided, the position of the task in ref is used instead
                (one-based). If the task cannot be found in ref, ValueError is
                thrown.

        Raises:
            ValueError: If ref is given but a task cannot be found in ref.
        """
        if ref:
            for task in tasks:
                index = ref.index(task)
                if index == -1:
                    raise ValueError(
                        "Task [{0}] cannot be found in {1}".format(
                            task,
                            ref,
                        )
                    )

                print(
                    "{0}) {1} (id: {2})".format(
                        index + 1,
                        task['title'],
                        task['id'],
                    )
                )
        else:
            for index, task in enumerate(tasks, 1):
                print(
                    "{0}) {1} (id: {2})".format(
                        index,
                        task['title'],
                        task['id'],
                    )
                )

    def _get_duplicated_items(self, items):
        seen = []
        duplicated = []
        for item in items:
            # TODO(htaihm): optimize this
            if item in seen:
                duplicated.append(item)
            else:
                seen.append(item)

        return duplicated

    def get_tasks_action(self, args):
        """Handle the subcommand get_tasks

        Print out the tasks for the task lists specified from the flags of the
        script.

        Args:
            args (argparse.Namespace): The flags of the script.
        """
        self.auth(args, use_credentials_cache=args.use_credentials_cache)

        tasklists_names = args.tasklist or []
        tasklists = self.get_tasklists()
        tasklists_to_show = []

        tasklists_map = {
            tasklist['title']: tasklist for tasklist in tasklists
        }

        if not tasklists_names:
            # assume all the task lists if no task lists are provided
            tasklists_to_show.extend(tasklists)
        else:
            for tasklist_name in tasklists_names:
                if tasklist_name not in tasklists_map:
                    _plain_error_logger.error(
                        "Task list [%s] does not exist. Avaliable task lists"
                            " are:",
                        tasklist_name,
                    )
                    for index, tasklist in enumerate(tasklists, 1):
                        if args.detail:
                            s = pprint.pformat(tasklist)
                        else:
                            s = tasklist['title']
                        _plain_logger.error("%d) %s", index, s)
                    return 1
                tasklists_to_show.append(tasklists_map[tasklist_name])

        for tasklist_to_show in tasklists_to_show:
            tasklist_id = tasklist_to_show['id']

            if args.detail:
                s = pprint.pformat(tasklist_to_show)
            else:
                s = "[%s]:" % tasklist_to_show['title']

            _plain_logger.info(
                "Tasks for the list %s",
                s,
            )
            tasks = self.get_tasks_from_tasklist(tasklist_id)
            for index, task in enumerate(tasks, 1):
                if args.detail:
                    s = pprint.pformat(task)
                else:
                    s = task['title']
                _plain_logger.info("%d) %s", index, s)
                if args.with_notes:
                    _plain_logger.info("Notes: %s", task.get('notes', ""))

        return 0


    def get_tasklists(self):
        """Retrieve the task lists of the user

        Returns:
            A list of dictionaries each represents a Tasklist resource. The
            exact representation is in
            https://developers.google.com/google-apps/tasks/v1/reference/tasklists#resource-representations
            Currently it is:

                {
                  "kind": "tasks#taskList",
                  "id": string,
                  "etag": string,
                  "title": string,
                  "updated": datetime,
                  "selfLink": string
                }
        """

        results = self.tasks_service.tasklists().list().execute()
        return results.get('items', [])

    def get_tasks_from_tasklist(self, tasklist_id):
        """Retrieves a list of tasks for a Tasklist

        Args:
            tasklist_id (string): The ID of the Tasklist.

        Returns:
            A list of dictionaries each represents a Task resource. The exact
            representation is in
            https://developers.google.com/google-apps/tasks/v1/reference/tasks#resource-representations
            Currently it is:

                {
                  "kind": "tasks#task",
                  "id": string,
                  "etag": etag,
                  "title": string,
                  "updated": datetime,
                  "selfLink": string,
                  "parent": string,
                  "position": string,
                  "notes": string,
                  "status": string,
                  "due": datetime,
                  "completed": datetime,
                  "deleted": boolean,
                  "hidden": boolean,
                  "links": [
                    {
                      "type": string,
                      "description": string,
                      "link": string
                    }
                  ]
                }
        """
        results = self.tasks_service.tasks().list(
            tasklist=tasklist_id
        ).execute()
        return results.get('items', [])


def parse_args(app_obj, cmd_args):
    description = __doc__.split("Copyright 2014")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
        parents=[tools.argparser],
    )
    parser.add_argument(
        "--no-credentials-cache",
        dest="use_credentials_cache",
        action="store_false",
        default=True,
        help="If specified, old credentials are not reused and you have to"
            " follow the instruction from this script to get the code every"
            " time you use this script.",
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    subparsers = parser.add_subparsers(
        title="Subcommands",
        description="Some description for subcommands",
    )

    get_tasks_description = """Print tasks for your task lists.

Examples:

    To print tasks for all of your task lists:

        $ sorno_gtasks.py get_tasks

    To print task only for your task list "list1" and "list2":

        $ sorno_gtasks.py get_tasks list1 list2

By default, get_tasks only prints the titles of your tasks. You can use
--with-notes option to print the notes as well. Use the --detail option to
show details.

Examples:

    To show the details for all tasks and all task lists.

        $ sorno_gtasks.py get_tasks --detail
    """

    parser_get_tasks = subparsers.add_parser(
        "get_tasks",
        help="Print your tasks",
        description=get_tasks_description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser_get_tasks.add_argument(
        "--with-notes",
        action="store_true",
        help="shows the notes for each task",
    )
    parser_get_tasks.add_argument(
        "--detail",
        action="store_true",
        help="see the details for all tasks and task lists",
    )
    parser_get_tasks.add_argument(
        "tasklist",
        nargs="*",
        help="The tasks in which to be printed out. If not specified, "
        " assume all tasks in all task lists.",
    )
    parser_get_tasks.set_defaults(func=app_obj.get_tasks_action)

    parser_copy_tasks = subparsers.add_parser(
        "copy_tasks",
        description="Copy tasks. Simply follow the prompt instructions to"
            " finish the action.",
        help="Copy tasks",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser_copy_tasks.set_defaults(func=app_obj.copy_tasks_action)

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _plain_logger, _plain_error_logger

    app = GoogleTasksConsoleApp()
    args = parse_args(app, sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)
    _plain_logger = loggingutil.create_plain_logger(
        "PLAIN",
        debug=args.debug,
    )
    _plain_error_logger = loggingutil.create_plain_logger(
        "PLAIN_ERROR",
        debug=args.debug,
        stdout=False,
    )
    args.func(args)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Gets stock quotes and other information for stock symbols.

The script can print real-time or close to real-time stock quotes, historical
quotes, and also fundamental ratios for the stock (company).


    Copyright 2014 Herman Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse

from cStringIO import StringIO

import csv
import datetime
import logging
import sys
import subprocess

from bs4 import BeautifulSoup
import requests
from sorno import loggingutil
from sorno.console.util import DataPrinter

_LOG = logging.getLogger(__name__)
_PLAIN_LOGGER = None  # will be created in main()
_PLAIN_ERROR_LOGGER = None  # will be created in main()


class StockApp(object):
    # https://code.google.com/p/yahoo-finance-managed/wiki/enumQuoteProperty
    FIELD_HEADERS = [
        ("name", "s"),
        ("price", "l1"),
        ("date", "d1"),
        ("time", "t1"),
        ("change", "c1"),
        ("changeInPercent", "c"),
        ("lastTradeRealtime", "k1"),
        ("changeRealtime", "c6"),
        ("changePercentRealtime", "k2"),
        ("open", "o"),
        ("high", "h"),
        ("low", "g"),
        ("volume", "v"),
        ("AverageDailyVolume", "a2"),
        # ("SharesOutstanding", "j2"), shares outstanding returns non-standard
        # csv value
        ("bidRealtime", "b3"),
        ("askRealtime", "b2"),
        ("YearLow", "j"),
        ("YearHigh", "k"),
        ("DilutedEPS", "e"),
        ("EPSEstimateCurrentYear", "e7"),
        ("EPSEstimateNextYear", "e8"),
        ("PERatio", "r"),
        ("PERatioRealtime", "r2"),
        ("ShortRatio", "s7"),
        ("DividendPayDate", "r1"),
        ("ExDividendDate", "q"),
    ]

    YAHOO_STOCK_QUOTE_CSV_API = "http://download.finance.yahoo.com/d/quotes.csv"
    YAHOO_STOCK_HISTORICAL_QUOTES_API = "http://ichart.yahoo.com/table.csv"
    YAHOO_STOCK_INSIDER_PURCHASES_PAGE_TEMPLATE = (
        "http://finance.yahoo.com/q/it?s=%s+Insider+Transactions"
    )

    QUANDL_STOCK_FUNDAMENTAL_API = "http://www.quandl.com/api/v1/datasets/OFDP/DMDRN_%s_ALLFINANCIALRATIOS.csv"

    def __init__(
        self,
        stock_symbol,
        print_fundamentals=False,
        print_history=False,
        print_price_quote=True,
        num_of_days_for_history=30,
        print_insider_purchases=False,
    ):
        self.stock_symbol = stock_symbol

        self.print_fundamentals = print_fundamentals
        self.print_history = print_history
        self.print_price_quote = print_price_quote

        self.num_of_days_for_history = num_of_days_for_history
        self.print_insider_purchases = print_insider_purchases

    def run(self):
        if self.print_price_quote:
            resp = requests.get(
                self.YAHOO_STOCK_QUOTE_CSV_API,
                params={
                    's': self.stock_symbol,
                    'f': "".join([h[1] for h in self.FIELD_HEADERS]),
                    'e': ".csv",
                },
            )
            _LOG.debug("URL: %s", resp.url)

            text = resp.text.strip()
            headers = [h[0] for h in self.FIELD_HEADERS]

            lines = [",".join(headers), text]
            reader = csv.DictReader(lines)
            # there is only one row in the result
            d = list(reader)[0]

            for header in headers:
                _PLAIN_LOGGER.info("%s:\t%s", header, d[header])

        if self.print_fundamentals:
            resp = requests.get(
                self.QUANDL_STOCK_FUNDAMENTAL_API % self.stock_symbol.upper()
            )

            text = resp.text

            reader = csv.DictReader(text.split("\n"))

            rows = list(reader)
            rows.sort(key=lambda r: r['Date'])

            DataPrinter(rows).print_result(DataPrinter.PRINT_STYLE_VERBOSE)

        if self.print_history:
            _PLAIN_LOGGER.info("")
            self.print_historical_stock_quotes()

        if self.print_insider_purchases:
            _PLAIN_LOGGER.info("")
            self.print_insider_purchase_entries()

    def print_historical_stock_quotes(self):
        today = datetime.datetime.today()
        from_date = today - datetime.timedelta(self.num_of_days_for_history)

        params = {
            's': self.stock_symbol,
            'a': from_date.month - 1,
            'b': from_date.day,
            'c': from_date.year,
            'd': today.month-1,
            'e': today.day,
            'f': today.year,
            'g': "d",  # daily quotes
            'ignore': ".csv",
        }

        resp = requests.get(
            self.YAHOO_STOCK_HISTORICAL_QUOTES_API,
            params=params,
        )

        _LOG.info("URL: %s", resp.url)

        for line in resp.text.split("\n"):
            _PLAIN_LOGGER.info(line.replace(",", "\t"))

    def print_insider_purchase_entries(self):
        resp = requests.get(
            self.YAHOO_STOCK_INSIDER_PURCHASES_PAGE_TEMPLATE % self.stock_symbol,
        )
        soup = BeautifulSoup(resp.text, "lxml")
        tables = soup.select("table.yfnc_tableout1")
        if len(tables) < 3:
            _PLAIN_ERROR_LOGGER.error("No insider purchases available")
            return 1

        insider_history_table = tables[2]

        headers = []
        rows = []
        for i, tr in enumerate(insider_history_table.select("tr table tr")):
            if i == 0:
                # process headers
                for th in tr.select("th"):
                    headers.append(th.get_text())
            else:
                # process data
                rows.append([td.get_text() for td in tr.select("td")])
        DataPrinter(
            rows,
            headers=headers,
            print_func=_PLAIN_LOGGER.info,
        ).print_result(DataPrinter.PRINT_STYLE_NICETABLE)


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2014")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    parser.add_argument(
        "--fundamentals",
        action="store_true",
        help="Print fundamentals data",
    )

    parser.add_argument(
        "--insider-purchases",
        action="store_true",
        help="Print insider purchases",
    )

    parser.add_argument(
        "--history",
        help="Print historical stock quotes",
        action="store_true",
    )

    parser.add_argument(
        "--no-price-quote",
        action="store_true",
        help="Not printing price quote of the stock",
    )

    parser.add_argument(
        "-n",
        "--num-of-days",
        help="Number of days to print for historical stock quotes",
        type=int,
        default=30,
    )

    parser.add_argument("stock_symbol", nargs="+")

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _PLAIN_LOGGER, _PLAIN_ERROR_LOGGER

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_LOG, debug=args.debug)
    _PLAIN_LOGGER = loggingutil.create_plain_logger("PLAIN")
    _PLAIN_ERROR_LOGGER = loggingutil.create_plain_logger("PLAIN_ERROR", stdout=False)

    for stock_symbol in args.stock_symbol:
        app = StockApp(
            stock_symbol,
            print_history=args.history,
            print_price_quote=not args.no_price_quote,
            print_fundamentals=args.fundamentals,
            num_of_days_for_history=args.num_of_days,
            print_insider_purchases=args.insider_purchases,
        )
        app.run()


if __name__ == '__main__':
    main()



#!/usr/bin/python
"""A command line client for Google Drive.

The API doc used to implement this is in
https://developers.google.com/drive/web/quickstart/quickstart-python

Currently, you can upload files with the script.

In order to use this script, please look at the "Using scripts involve Google
App API" section of the sorno-py-scripts README (can be found in
https://github.com/hermantai/sorno-scripts/tree/master/sorno-py-scripts). The
API needed for this script is "Drive API" with the scope
'https://www.googleapis.com/auth/drive.readonly'

Examples:

    You can upload files by running:

        $ sorno_gdrive.py upload <file1> [file2, file3...]


    Copyright 2014 Herman Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import argparse
import logging
import os
import pprint
import subprocess
import sys

import apiclient
from apiclient.discovery import build
from apiclient.http import MediaFileUpload

import httplib2

from oauth2client.client import OAuth2WebServerFlow
from oauth2client.file import Storage

from sorno import loggingutil


_LOG = logging.getLogger(__name__)

# Check https://developers.google.com/drive/scopes for all available scopes
OAUTH_SCOPE = 'https://www.googleapis.com/auth/drive'

# Redirect URI for installed apps
REDIRECT_URI = 'urn:ietf:wg:oauth:2.0:oob'

CREDENTIALS_FILE = os.path.expanduser("~/.sorno_gdrive-google-drive-api.cred")


class App(object):
    def __init__(self):
        self.drive_service = None

    def auth(self, use_credentials_cache=True):
        # Copy your credentials from the console
        client_id = os.getenv('GOOGLE_APP_PROJECT_CLIENT_ID')
        client_secret = os.getenv('GOOGLE_APP_PROJECT_CLIENT_SECRET')

        if not client_id:
            print(
                "Please set the environment variable"
                " GOOGLE_APP_PROJECT_CLIENT_ID"
            )
            sys.exit(1)

        if not client_secret:
            print(
                "Please set the environment variable"
                " GOOGLE_APP_PROJECT_CLIENT_SECRET"
            )
            sys.exit(1)

        # Run through the OAuth flow and retrieve credentials
        flow = OAuth2WebServerFlow(client_id, client_secret, OAUTH_SCOPE,
                                   redirect_uri=REDIRECT_URI)

        # Indicate we need the user to grant us permissions and give the auth code or
        # not
        need_get_code = True

        if os.path.exists(CREDENTIALS_FILE) and use_credentials_cache:
            storage = Storage(CREDENTIALS_FILE)
            credentials = storage.get()
            print("Use old credentials")
            need_get_code = False

        if need_get_code:
            authorize_url = flow.step1_get_authorize_url()
            print 'Go to the following link in your browser: ' + authorize_url
            code = raw_input('Enter verification code: ').strip()

            credentials = flow.step2_exchange(code)

            storage = Storage(CREDENTIALS_FILE)
            storage.put(credentials)

        # Create an httplib2.Http object and authorize it with our credentials
        http = httplib2.Http()
        http = credentials.authorize(http)

        self.drive_service = build('drive', 'v2', http=http)

    def upload_action(self, args):
        self.auth(args.use_credentials_cache)
        self.upload_files(args.files)

    def upload_files(self, filepaths):
        _LOG.info("Will upload files: %s" , ", ".join(filepaths))

        for filepath in filepaths:
            _LOG.info("Upload file: %s", filepath)
            self.ensure_mimetype_exists(filepath)
            media = MediaFileUpload(
                filepath,
            )

            file_title = os.path.basename(filepath)
            response = self.drive_service.files().insert(
                media_body=media,
                body={
                    'title': file_title,
                },
            ).execute()

            pprint.pprint(response)

    def download_action(self, args):
        _LOG.error("Not implemented, yet")
        sys.exit(1)

    def ensure_mimetype_exists(self, filepath):
        import mimetypes
        if mimetypes.guess_type(filepath)[0]:
            return

        _LOG.warn(
            "Cannot detect mimetype for %s, assume it's text/plain.",
            filepath,
        )

        ext = os.path.splitext(filepath)[1]
        # works most of the time, if not, too bad
        _LOG.info("Add type text/plain for extension %s", ext)
        mimetypes.add_type('text/plain', ext, strict=True)
        assert mimetypes.guess_type(filepath)


def main():
    app = App()
    args = parse_args(app, sys.argv[1:])

    loggingutil.setup_logger(_LOG, debug=args.debug)
    args.func(args)


def parse_args(app_obj, cmd_args):
    description = __doc__.split("Copyright 2014")[0].strip()
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--no-credentials-cache",
        dest="use_credentials_cache",
        action="store_false",
        default=True,
        help="If specified, old credentials are not reused and you have to"
            " follow the instruction from this script to get the code every"
            " time you use this script.",
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    subparsers = parser.add_subparsers(title="Subcommands")

    parser_upload = subparsers.add_parser("upload")
    parser_upload.add_argument(
        "files",
        nargs="+",
        help="Local filepaths for files to be uploaded",
    )
    parser_upload.set_defaults(func=app_obj.upload_action)

    parser_download = subparsers.add_parser("download")
    parser_download.set_defaults(func=app_obj.download_action)

    args = parser.parse_args(cmd_args)
    return args


if __name__ == '__main__':
    main()




#!/usr/bin/env python
"""A script version of Dropbox

Provides utilities to work with Dropbox just like the official dropbox cli
(http://www.dropboxwiki.com/tips-and-tricks/using-the-official-dropbox-command-line-interface-cli),
but in a script instead of a REPL way. sorno_dropbox also has higher level
features like copying directories recursively. Before you use this script, you
need to export the environment variables DROPBOX_APP_KEY and
DROPBOX_APP_SECRET for the app key and app secret of Dropbox. You can find
these at http://www.dropbox.com/developers/apps. For simplicity sake, just
give the app key "Full Dropbox" permission.


    Copyright 2014 Herman Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import os
import pprint
import re
import sys

from dropbox import client, rest, session
import six
from sorno import loggingutil


# Fill in your consumer key and secret below or set the corresponding
# environment variables.
# You can find these at http://www.dropbox.com/developers/apps
DROPBOX_APP_KEY = None  # Full Dropbox permission
DROPBOX_APP_SECRET = None
DROPBOX_TOKEN_FILE = "/tmp/dropbox_token"

_LOG = logging.getLogger(__name__)
_PLAIN_LOGGER = None  # will be created in main()


def command(login_required=True):
    """a decorator for handling authentication and exceptions"""
    def decorate(f):
        def wrapper(self, *args):
            if login_required and self.api_client is None:
                _PLAIN_LOGGER.info("Need to login first")
                self.login()
                if self.api_client is None:
                    _PLAIN_LOGGER.error("Fail to login")
                    return

            try:
                return f(self, *args)
            except rest.ErrorResponse as e:
                msg = e.user_error_msg or str(e)
                _LOG.exception(msg)

        wrapper.__doc__ = f.__doc__
        return wrapper
    return decorate


class DropboxApp(object):
    def __init__(
        self,
        app_key,
        app_secret,
    ):
        self.app_key = app_key
        self.app_secret = app_secret

        self.api_client = None
        try:
            serialized_token = open(DROPBOX_TOKEN_FILE).read()
            if serialized_token.startswith('oauth1:'):
                access_key, access_secret = serialized_token[len('oauth1:'):].split(':', 1)
                sess = session.DropboxSession(self.app_key, self.app_secret)
                sess.set_token(access_key, access_secret)
                self.api_client = client.DropboxClient(sess)
                _LOG.debug("[loaded OAuth 1 access token]")
            elif serialized_token.startswith('oauth2:'):
                access_token = serialized_token[len('oauth2:'):]
                self.api_client = client.DropboxClient(access_token)
                _LOG.debug("[loaded OAuth 2 access token]")
            else:
                _LOG.warn("Malformed access token in %r.", DROPBOX_TOKEN_FILE)
        except IOError:
            pass # don't worry if it's not there

    @command(login_required=True)
    def do_ls(self, args):
        dirpath = "" if not args.dirpath else args.dirpath
        fileslist = self.ls(dirpath)
        if fileslist is None:
            _PLAIN_LOGGER.error(
                "The dirpath [%s] does not exist.",
                dirpath,
            )
            return 1

        for name in fileslist:
            _PLAIN_LOGGER.info(name)

        return 0

    def ls(self, dirpath):
        """List files in current remote directory

        Returns:
            A list of file/directory names in strings under the given dirpath.
            Return None if dirpath does not exist.
        """
        try:
            resp = self.api_client.metadata(dirpath)
            _LOG.debug(resp)
            if resp.get('is_deleted'):
                return None
        except rest.ErrorResponse as e:
            if e.status == 404:
                return None
            raise

        if 'contents' in resp:
            names = []
            for f in resp['contents']:
                name = os.path.basename(f['path'])
                names.append(name)
            return names
        return None

    def is_exists(self, filepath):
      try:
          resp = self.api_client.metadata(filepath)
          _LOG.debug(resp)
          return not resp.get('is_deleted')
      except rest.ErrorResponse as e:
          if e.status == 404:
              return False
          raise

    @command(login_required=False)
    def do_login(self):
        self.login()

    def login(self):
        """log in to a Dropbox account"""
        flow = client.DropboxOAuth2FlowNoRedirect(self.app_key, self.app_secret)
        authorize_url = flow.start()
        _PLAIN_LOGGER.info("1. Go to: " + authorize_url + "\n")
        _PLAIN_LOGGER.info("2. Click \"Allow\" (you might have to log in first).\n")
        _PLAIN_LOGGER.info("3. Copy the authorization code.\n")
        code = six.moves.input("Enter the authorization code here: ").strip()

        try:
            access_token, user_id = flow.finish(code)
            _LOG.debug("User id is %s", user_id)
        except rest.ErrorResponse, e:
            _PLAIN_LOGGER.error('Error: %s', e)
            return

        with open(DROPBOX_TOKEN_FILE, 'w') as f:
            f.write('oauth2:' + access_token)
        _LOG.info(
            "The access token is written to the file %s",
            DROPBOX_TOKEN_FILE,
        )
        self.api_client = client.DropboxClient(access_token)
        _LOG.debug("Client info: %s", self.api_client.account_info())

    @command(login_required=True)
    def do_copydir(self, args):
        try:
            self.copydir(args.from_path, args.to_path)
            return 0
        except IOError as e:
            _PLAIN_LOGGER.error("%s", e)
            return 1

    def copydir(self, src_dir, dest_dir):
      # create dest dir
      src_dir = src_dir.rstrip('/')

      if not self.is_exists(dest_dir):
          raise IOError("Dest dir [%s] does not exist!" % dest_dir)
          return

      dest_dir = os.path.join(dest_dir, os.path.basename(src_dir))
      sys.stdout.write("dest: " + dest_dir + "\n")

      if not self.is_exists(dest_dir):
        self.prompt("Do you want to create directory [%s]?" % dest_dir)
        self.mkdir(dest_dir)

      # do the copying
      for root, dirs, files in os.walk(src_dir):
        for f in files:
          src_filepath = os.path.join(root, f)
          dest_filepath = os.path.join(dest_dir, src_filepath[len(src_dir):].lstrip('/'))
          sys.stdout.write("put %s to %s\n" % (src_filepath, dest_filepath))
          self.put(src_filepath, dest_filepath)
        for d in dirs:
          src_filepath = os.path.join(root, d)
          dest_filepath = os.path.join(dest_dir, src_filepath[len(src_dir):].lstrip('/'))
          sys.stdout.write("mkdir %s\n" % dest_filepath)
          self.mkdir(dest_filepath)

    @command()
    def do_get(self, args):
        if args.to_path:
            to_path = args.to_path
        else:
            to_path = os.path.join('.', os.path.basename(args.from_path))

        self.get(args.from_path, to_path)
        return 0

    def get(self, from_path, to_path):
        """
        Copy file from Dropbox to local file and print out the metadata.

        Examples:
        sorno_dropbox.py get file.txt ~/dropbox-file.txt
        """
        f, metadata = self.api_client.get_file_and_metadata(from_path)

        _PLAIN_LOGGER.info('Metadata: %s', pprint.pformat(metadata))

        with open(to_path, "wb") as to_file:
            to_file.write(f.read())

        f.close()

    @command()
    def do_mkdir(self, args):
        self.mkdir(args.dirpath)
        return 0

    def mkdir(self, dirpath):
        """create a new directory"""
        self.api_client.file_create_folder(dirpath)

    @command()
    def do_put(self, args):
        self.put(args.from_path, args.to_path, overwrite=args.overwrite)
        return 0

    def put(self, from_path, to_path, overwrite=False):
        """Copy local file to Dropbox

        Examples:
        sorno_dropbox.py put ~/test.txt dropbox-copy-test.txt
        """
        from_file = open(from_path, "rb")
        self.api_client.put_file(to_path, from_file, overwrite=overwrite)

    @command(login_required=True)
    def do_search(self, args):
        dirpath = "" if not args.dirpath else args.dirpath
        results = self.search(dirpath, args.query)
        for r in results:
            if args.detail:
                _PLAIN_LOGGER.info(pprint.pformat(r))
            else:
                _PLAIN_LOGGER.info(r['path'])

        return 0

    def search(self, dirpath, query):
        results = self.api_client.search(dirpath, query)
        return results

    def prompt(self, msg):
        ans = six.moves.input(msg)
        if ans.lower() in ('y', 'yes'):
            return
        else:
            raise Exception("aborted!")


def parse_args(app_obj, cmd_args):
    description = __doc__.split("Copyright 2014")[0].rstrip()
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    subparsers = parser.add_subparsers(
        title="Commands",
    )

    #
    # parser_ls
    #

    ls_help = "List the content of a directory"
    parser_ls = subparsers.add_parser(
        "ls",
        help=ls_help,
        description=ls_help,
    )
    parser_ls.add_argument(
        "dirpath",
        nargs="?",
        help="The path of the directory in dropbox. For the root directory,"
        " use an empty string",
    )
    parser_ls.set_defaults(func=app_obj.do_ls)

    #
    # parser_copydir
    #

    copydir_help = "Copy a directory to dropbox recursively"
    parser_copydir = subparsers.add_parser(
        "copydir",
        help=copydir_help,
        description=copydir_help,
    )
    parser_copydir.add_argument(
        "from_path",
    )
    parser_copydir.add_argument(
        "to_path",
    )
    parser_copydir.set_defaults(func=app_obj.do_copydir)

    #
    # parser_get
    #

    get_help = (
        """
        Copy file from Dropbox to local file and print out the metadata.

        Examples:
        get file.txt ~/dropbox-file.txt
        """
    )
    parser_get = subparsers.add_parser(
        "get",
        help=get_help,
        description=get_help,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser_get.add_argument(
        "from_path",
        help="The location in dropbox",
    )
    parser_get.add_argument(
        "to_path",
        nargs="?",
        help=re.sub(
            r"\s+",
            " ",
            """
            The local path. If if you skip this, the filename of the from_path
            is used instead and will be downloaded to your current directory.
            """,
        ).strip(),
    )

    parser_get.set_defaults(func=app_obj.do_get)

    #
    # parser_put
    #

    put_help = (
        "Copy. Examples: put ~/test.txt dropbox-copy-test.txt"
    )
    parser_put = subparsers.add_parser(
        "put",
        help=put_help,
        description=put_help,
    )
    parser_put.add_argument(
        "from_path",
        help="The local path",
    )
    parser_put.add_argument(
        "to_path",
        help="The location in dropbox",
    )
    parser_put.add_argument(
        "--overwrite",
        help="Overwrite the existing file or not. If false, a file"
        " with a new name is created instead of the specified name"
        " in to_path",
        action="store_true",
    )
    parser_put.set_defaults(func=app_obj.do_put)

    #
    # parser_search
    #

    search_help = "Search in dropbox"
    parser_search = subparsers.add_parser(
        "search",
        help=search_help,
        description=search_help,
    )
    parser_search.add_argument(
        "--dirpath",
        help="The directory to e searched from. By default it searches from"
            + " the root."
    )
    parser_search.add_argument(
        "--detail",
        help="If true, the whole details of the results are printed out",
        action="store_true",
    )
    parser_search.add_argument("query")
    parser_search.set_defaults(func=app_obj.do_search)

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _PLAIN_LOGGER

    app = DropboxApp(
        DROPBOX_APP_KEY or os.getenv('DROPBOX_APP_KEY'),
        DROPBOX_APP_SECRET or os.getenv('DROPBOX_APP_SECRET'),
    )
    args = parse_args(app, sys.argv[1:])

    loggingutil.setup_logger(_LOG, debug=args.debug)
    _PLAIN_LOGGER = loggingutil.create_plain_logger(
        "PLAIN",
        debug=args.debug,
    )
    return args.func(args)


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
"""Replaces constants with literal values for a thrift file except for the
declaration. This is mainly for thrift compilers which cannot handle constants
within lists or other collection structures.


    Copyright 2014 Herman Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import re
import sys


class ThriftConstReplacerApp(object):
    def run(self, filepath):
        replacer = ThriftConstReplacer()
        with open(filepath) as f:
            for line in f:
                print(replacer.parse_line(line), end="")


class ThriftConstReplacer(object):
    def __init__(self):
        self._consts = {}
        self._coming_line_is_for_this_const = None

    def parse_line(self, line):
        """
        Parse a line, then replace the constants with literal values as
        needed. If a constant declaration is parsed, the constant is stored,
        so that the literal value can be used to replace other lines later.

        All constants can be retrieved using get_consts.
        """
        # 1. Check if we have to take care of previous line constant
        # declaration
        if self._coming_line_is_for_this_const:
            value = self._line_without_semicolon_stripped(line)
            self._consts[self._coming_line_is_for_this_const] = value
            self._coming_line_is_for_this_const = None
            return line

        # 2. Replace constants with the corresponding value
        tokens = self._parse_line(line)
        for token in tokens:
            if token.s in self._consts:
                token.s = self._consts[token.s]

        line = "".join([t.s for t in tokens])

        # 3. Handle constant declaration
        tokens = self._line_without_semicolon_stripped(line).split()

        # A constant declaration is like (make sure we don't store constants
        # which are collections) :
        #
        #   const string a = "abc"
        # or
        #   const string a =
        #       "long value"
        #
        # so it has at least four tokens.
        if len(tokens) >= 4:
            if tokens[0] == "const" and (
                tokens[3] == "=" and "<" not in tokens[1]
            ):
                const = tokens[2]
                if len(tokens) >= 5:
                    # there is at least one value on the right size of the
                    # equal sign, so get the value on the right side of the
                    # equal sign
                    value = self._line_without_semicolon_stripped(
                        line
                    ).split('=', 1)[1].strip()
                    self._consts[const] = value
                else:
                    # assume the next line is the value for the constant
                    self._coming_line_is_for_this_const = const

        return line

    def get_consts(self):
        return self._consts

    def _line_without_semicolon_stripped(self, line):
        return line.strip().rstrip(';').strip()

    def _parse_line(self, line):
        tokens = []

        while line:
            m = NonWord.REGEX.match(line)
            if m:
                tokens.append(NonWord(m.group(1)))
                line = NonWord.REGEX.sub("", line, count=1)
            else:
                m = Word.REGEX.match(line)
                tokens.append(Word(m.group(1)))
                line = Word.REGEX.sub("", line, count=1)

        return tokens


class Word(object):
    """
    matches any alphanumeric character and the underscore; this is equivalent
    to the set [a-zA-Z0-9_].
    """
    REGEX = re.compile(r"(\w+)")

    def __init__(self, s):
        self.s = s


class NonWord(object):
    REGEX = re.compile("(\W+)")

    def __init__(self, s):
        self.s = s


def parse_args(cmd_args):
    description = """
Replace constants with literal values for a thrift file except for the
declaration. This is mainly for thrift compilers which cannot handle constants
within lists or other collection structures.
    """
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "file"
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    app = ThriftConstReplacerApp()
    app.run(args.file)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
sorno_appcache_generator.py


   Copyright 2014 Herman Tai

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
from argparse import ArgumentParser
import time

import logging
import os

import sys
import subprocess


_LOG = logging.getLogger(__name__)
_PLAIN_LOGGER = None  # will be created in main()


_APPCACHE_TEMPLATE = """
CACHE MANIFEST
# version {date} 01

CACHE:
{files_list}

NETWORK:
*
""".strip()

class App(object):
    def __init__(
        self,
        web_app_root,
    ):
        self.web_app_root = web_app_root

    def run(self):
        _LOG.debug("Generate appcache with root dir: %s", self.web_app_root)

        date = time.strftime("%Y-%m-%d")

        filepaths = []

        web_app_root = os.path.realpath(self.web_app_root)
        _LOG.debug("Walk dir: %s", web_app_root)

        for root, dirs, files in os.walk(web_app_root):
            dir_under_web_app_root = os.path.realpath(
                root
            ).replace(
                web_app_root, ""
            ).lstrip(os.path.sep)

            _LOG.debug("dir_under_web_app_root is %s", dir_under_web_app_root)

            for f in files:
                if not self.skip_file(f):
                    filepath = os.path.join(dir_under_web_app_root, f)
                    filepaths.append(filepath)

        filepaths.sort()
        files_list = os.linesep.join(filepaths)

        print(_APPCACHE_TEMPLATE.format(**locals()))

    def skip_file(self, filename):
        return filename.startswith(".")
#
# logger-related functions
#

def setup_logger(
    logger,
    debug=False,
    stdout=False,
    log_to_file=None,
    add_thread_id=False,
    logging_level=None,
    use_path=False,
):
    if logging_level is None:
        if debug:
            logging_level = logging.DEBUG
        else:
            logging_level = logging.INFO

    formatter = create_logging_formatter(
        add_thread_id=add_thread_id,
        use_path=use_path
    )
    hdlr = create_stream_handler(
        formatter=formatter,
        stdout=stdout,
    )
    logger.handlers = []  # clear the existing handlers
    logger.addHandler(hdlr)
    logger.setLevel(logging_level)

    if log_to_file is not None:
        init_command = 'mkdir -p %s' % os.path.dirname(log_to_file)

        subprocess.check_call(init_command, shell=True)

        hdlr = TimedRotatingFileHandler(
            log_to_file,
            when="midnight",
            interval=1,
        )
        hdlr.setFormatter(formatter)
        logger.addHandler(hdlr)


def create_plain_logger(logger_name, stdout=True):
    plain_logger = logging.getLogger(logger_name)
    plain_logger.propagate = False
    plain_logger.setLevel(logging.INFO)

    if stdout:
        out = sys.stdout
    else:
        out = sys.stderr

    handler = logging.StreamHandler(stream=out)
    handler.setFormatter(
        logging.Formatter(
            fmt="%(message)s",
            datefmt="%Y",  # does not matter
        )
    )

    plain_logger.addHandler(handler)
    return plain_logger


def create_logging_formatter(add_thread_id=False, use_path=False):
    format_str = "%(asctime)s"

    if add_thread_id:
        format_str += " thread:%(thread)s"

    format_str += " %(levelname)s "

    if use_path:
        format_str += "%(pathname)s"
    else:
        format_str += "%(name)s"

    format_str += ":%(lineno)s: %(message)s"

    detail_formatter = logging.Formatter(
        fmt=format_str,
        datefmt="%Y-%m-%d %H:%M:%S"
    )
    return detail_formatter


def create_stream_handler(formatter=None, stdout=False):
    if formatter is None:
        formatter = create_logging_formatter()

    if stdout:
        stream = sys.stdout
    else:
        stream = sys.stderr

    hdlr = logging.StreamHandler(stream=stream)
    hdlr.setFormatter(formatter)
    return hdlr

#
# end of logger-related functions
#

def parse_args(cmd_args):
    description = """
Generate an appcache file to be used for html5 application cache of a web
application. The goal is to make the whole web app cached, so the app can be
run offline.

Synopsis:
sorno_appcache_generator.py <web app directory>

It prints out the appcache content, so you should redirect the standard output
to a file for your web app. The appcache file is assumed to be put into the
same folder as the web app root directory.
    """
    parser = ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    parser.add_argument(
        "web_app_dir",
        help="The root directory of the web app",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _PLAIN_LOGGER

    args = parse_args(sys.argv[1:])

    setup_logger(_LOG, debug=args.debug)
    _PLAIN_LOGGER = create_plain_logger("PLAIN")

    app = App(
        args.web_app_dir,
    )
    app.run()


if __name__ == '__main__':
    main()
